@@ -534,7 +534,7 @@ void LocationFilter::LoadChild(const DataNode &child, const set<const System *>
 	int valueIndex = 1 + isNot;
 	const string &key = child.Token(valueIndex - 1);
 	if(key == "not" || key == "neighbor")
-		child.PrintTrace("Error: Skipping unsupported use of 'not' and 'neighbor'."
+		child.PrintTrace("Skipping unsupported use of 'not' and 'neighbor'."
 			" These keywords must be nested if used together.");
 	else if(key == "planet")
 	{
