@@ -17,8 +17,9 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 
 #include "DataNode.h"
 #include "DataWriter.h"
-#include "Dialog.h"
+#include "DialogPanel.h"
 #include "DistanceMap.h"
+#include "Endpoint.h"
 #include "text/Format.h"
 #include "GameData.h"
 #include "Government.h"
@@ -73,9 +74,9 @@ namespace {
 	void ParseMixedSpecificity(const DataNode &node, string &&kind, int expected)
 	{
 		if(node.Size() >= expected + 1)
-			node.PrintTrace("Warning: use a location filter to choose from multiple " + kind + "s:");
+			node.PrintTrace("Use a location filter to choose from multiple " + kind + "s:");
 		if(node.HasChildren())
-			node.PrintTrace("Warning: location filter ignored due to use of explicit " + kind + ":");
+			node.PrintTrace("Location filter ignored due to use of explicit " + kind + ":");
 	}
 
 	string TriggerToText(Mission::Trigger trigger)
@@ -130,21 +131,21 @@ void Mission::Load(const DataNode &node, const ConditionsStore *playerConditions
 	// All missions need a name.
 	if(node.Size() < 2)
 	{
-		node.PrintTrace("Error: No name specified for mission:");
+		node.PrintTrace("No name specified for mission:");
 		return;
 	}
 	// If a mission object is "loaded" twice, that is most likely an error (e.g.
 	// due to a plugin containing a mission with the same name as the base game
 	// or another plugin). This class is not designed to allow merging or
 	// overriding of mission data from two different definitions.
-	if(!name.empty())
+	if(!trueName.empty())
 	{
-		node.PrintTrace("Error: Duplicate definition of mission:");
+		node.PrintTrace("Duplicate definition of mission:");
 		return;
 	}
-	name = node.Token(1);
-	if(!DataNode::IsConditionName(name))
-		node.PrintTrace("Error: Invalid mission name");
+	trueName = node.Token(1);
+	if(!DataNode::IsConditionName(trueName))
+		node.PrintTrace("Invalid mission name:");
 
 	for(const DataNode &child : node)
 	{
@@ -187,7 +188,7 @@ void Mission::Load(const DataNode &node, const ConditionsStore *playerConditions
 				if(!ParseContraband(grand))
 					grand.PrintTrace("Skipping unrecognized attribute:");
 				else
-					grand.PrintTrace("Warning: Deprecated use of \"stealth\" and \"illegal\" as a child of \"cargo\"."
+					grand.PrintTrace("Deprecated use of \"stealth\" and \"illegal\" as a child of \"cargo\"."
 						" They are now mission-level properties:");
 			}
 		}
@@ -241,6 +242,8 @@ void Mission::Load(const DataNode &node, const ConditionsStore *playerConditions
 			location = JOB_BOARD;
 		else if(key == "entering")
 			location = ENTERING;
+		else if(key == "transition")
+			location = TRANSITION;
 		else if(key == "repeat")
 			repeat = (child.Size() == 1 ? 0 : static_cast<int>(child.Value(1)));
 		else if(key == "clearance")
@@ -289,7 +292,7 @@ void Mission::Load(const DataNode &node, const ConditionsStore *playerConditions
 			{
 				LocationFilter loaded(child, visitedSystems, visitedPlanets);
 				if(loaded.IsEmpty())
-					child.PrintTrace("Error: The \"complete at\" filter must not be empty. Ignoring this filter.");
+					child.PrintTrace("The \"complete at\" filter must not be empty. Ignoring this filter.");
 				else
 					completionFilter = loaded;
 			}
@@ -324,6 +327,8 @@ void Mission::Load(const DataNode &node, const ConditionsStore *playerConditions
 			substitutions.Load(child, playerConditions);
 		else if(key == "npc")
 			npcs.emplace_back(child, playerConditions, visitedSystems, visitedPlanets);
+		else if(key == "timer" && hasValue)
+			timers.emplace_back(child, playerConditions, visitedSystems, visitedPlanets);
 		else if(key == "on" && hasValue && child.Token(1) == "enter")
 		{
 			// "on enter" nodes may either name a specific system or use a LocationFilter
@@ -336,6 +341,18 @@ void Mission::Load(const DataNode &node, const ConditionsStore *playerConditions
 			else
 				genericOnEnter.emplace_back(child, playerConditions, visitedSystems, visitedPlanets);
 		}
+		else if(key == "on" && hasValue && child.Token(1) == "land")
+		{
+			// "on land" nodes may either name a specific planet or use a LocationFilter
+			// to control the triggering planet.
+			if(child.Size() >= 3)
+			{
+				MissionAction &action = onLand[GameData::Planets().Get(child.Token(2))];
+				action.Load(child, playerConditions, visitedSystems, visitedPlanets);
+			}
+			else
+				genericOnLand.emplace_back(child, playerConditions, visitedSystems, visitedPlanets);
+		}
 		else if(key == "on" && hasValue)
 		{
 			static const map<string, Trigger> trigger = {
@@ -381,7 +398,8 @@ void Mission::Load(const DataNode &node, const ConditionsStore *playerConditions
 	}
 
 	if(displayName.empty())
-		displayName = name;
+		displayName = trueName;
+	hasTrackedNpcs = ranges::any_of(npcs, [](const NPC &npc) { return npc.GetPersonality().IsTracked(); });
 }
 
 
@@ -390,7 +408,7 @@ void Mission::Load(const DataNode &node, const ConditionsStore *playerConditions
 // is already "instantiated," so only a subset of the data must be saved.
 void Mission::Save(DataWriter &out, const string &tag) const
 {
-	out.Write(tag, name);
+	out.Write(tag, trueName);
 	out.BeginChild();
 	{
 		out.Write("name", displayName);
@@ -416,8 +434,8 @@ void Mission::Save(DataWriter &out, const string &tag) const
 		auto saveColor = [&out](const ExclusiveItem<Color> &color, string tokenName) noexcept -> void {
 			if(!color->IsLoaded())
 				return;
-			if(!color->Name().empty())
-				out.Write("color", tokenName, color->Name());
+			if(!color->TrueName().empty())
+				out.Write("color", tokenName, color->TrueName());
 			else
 			{
 				const float *rgba = color->Get();
@@ -463,6 +481,8 @@ void Mission::Save(DataWriter &out, const string &tag) const
 			out.Write("job");
 		else if(location == ENTERING)
 			out.Write("entering");
+		else if(location == TRANSITION)
+			out.Write("transition");
 		if(!clearance.empty())
 		{
 			out.Write("clearance", clearance);
@@ -537,6 +557,8 @@ void Mission::Save(DataWriter &out, const string &tag) const
 
 		for(const NPC &npc : npcs)
 			npc.Save(out);
+		for(const MissionTimer &timer : timers)
+			timer.Save(out);
 
 		// Save all the actions, because this might be an "available mission" that
 		// has not been received yet but must still be included in the saved game.
@@ -549,6 +571,12 @@ void Mission::Save(DataWriter &out, const string &tag) const
 		for(const MissionAction &action : genericOnEnter)
 			if(!didEnter.contains(&action))
 				action.Save(out);
+		for(const auto &[planet, action] : onLand)
+			if(!didLand.contains(&action))
+				action.Save(out);
+		for(const MissionAction &action : genericOnLand)
+			if(!didLand.contains(&action))
+				action.Save(out);
 	}
 	out.EndChild();
 }
@@ -570,7 +598,17 @@ const EsUuid &Mission::UUID() const noexcept
 
 
 
-const string &Mission::Name() const
+// Get the internal name used for this mission. This name is unique and is
+// never modified by string substitution, so it can be used in condition
+// variables, etc.
+const string &Mission::TrueName() const
+{
+	return trueName;
+}
+
+
+
+const string &Mission::DisplayName() const
 {
 	return displayName;
 }
@@ -657,13 +695,19 @@ bool Mission::IsValid() const
 	for(auto &&it : onEnter)
 		if(!it.first->IsValid() || !it.second.Validate().empty())
 			return false;
+	for(const auto &[planet, action] : onLand)
+		if(!planet->IsValid() || !action.Validate().empty())
+			return false;
 	for(auto &&it : actions)
 		if(!it.second.Validate().empty())
 			return false;
-	// Generic "on enter" may use a LocationFilter that exclusively references invalid content.
+	// Generic "on enter" or "on land" may use a LocationFilter that exclusively references invalid content.
 	for(auto &&action : genericOnEnter)
 		if(!action.Validate().empty())
 			return false;
+	for(const auto &action : genericOnLand)
+		if(!action.Validate().empty())
+			return false;
 	if(!clearanceFilter.IsValid())
 		return false;
 
@@ -776,18 +820,51 @@ const set<const System *> &Mission::UnmarkedSystems() const
 
 
 
-void Mission::Mark(const System *system) const
+const set<const System *> &Mission::TrackedSystems() const
 {
-	markedSystems.insert(system);
-	unmarkedSystems.erase(system);
+	return trackedSystems;
 }
 
 
 
-void Mission::Unmark(const System *system) const
+void Mission::RecalculateTrackedSystems()
 {
-	if(markedSystems.erase(system))
-		unmarkedSystems.insert(system);
+	if(!hasTrackedNpcs)
+		return;
+
+	trackedSystems.clear();
+	for(const NPC &npc : npcs)
+	{
+		if(!npc.GetPersonality().IsTracked())
+			continue;
+		for(const auto &ship : npc.Ships())
+			if(!ship->IsDestroyed() && ship->GetSystem())
+				trackedSystems.insert(ship->GetSystem());
+	}
+	// If all tracked NPCs are destroyed, then we can ignore
+	// future calls to recalculate the tracked systems.
+	if(trackedSystems.empty())
+		hasTrackedNpcs = false;
+}
+
+
+
+void Mission::Mark(const set<const System *> &systems) const
+{
+	for(const System *system : systems)
+	{
+		markedSystems.insert(system);
+		unmarkedSystems.erase(system);
+	}
+}
+
+
+
+void Mission::Unmark(const set<const System *> &systems) const
+{
+	for(const System *system : systems)
+		if(markedSystems.erase(system))
+			unmarkedSystems.insert(system);
 }
 
 
@@ -911,7 +988,7 @@ bool Mission::CanOffer(const PlayerInfo &player, const shared_ptr<Ship> &boardin
 		if(!sourceFilter.Matches(*boardingShip))
 			return false;
 	}
-	else if(location == ENTERING)
+	else if(location == ENTERING || location == TRANSITION)
 	{
 		if(!sourceFilter.Matches(player.GetSystem()))
 			return false;
@@ -931,7 +1008,7 @@ bool Mission::CanOffer(const PlayerInfo &player, const shared_ptr<Ship> &boardin
 	if(!toFail.IsEmpty() && toFail.Test())
 		return false;
 
-	if(repeat && player.Conditions().Get(name + ": offered") >= repeat)
+	if(repeat && player.Conditions().Get(trueName + ": offered") >= repeat)
 		return false;
 
 	bool isFailed = IsFailed();
@@ -1026,6 +1103,10 @@ bool Mission::IsSatisfied(const PlayerInfo &player) const
 	for(const NPC &npc : npcs)
 		if(!npc.HasSucceeded(player.GetSystem()))
 			return false;
+	// All non-optional timers must be complete.
+	for(const MissionTimer &timer : timers)
+		if(!timer.IsOptional() && !timer.IsComplete())
+			return false;
 
 	// If any of the cargo for this mission is being carried by a ship that is
 	// not in this system, the mission cannot be completed right now.
@@ -1163,15 +1244,15 @@ bool Mission::Do(Trigger trigger, PlayerInfo &player, UI *ui, const shared_ptr<S
 	if(trigger == STOPOVER)
 	{
 		// If this is not one of this mission's stopover planets, or if it is
-		// not the very last one that must be visited, do nothing.
+		// not the very last one that must be visited, see if a landing action can trigger instead.
 		auto it = stopovers.find(player.GetPlanet());
 		if(it == stopovers.end())
-			return false;
+			return Land(player.GetPlanet(), player, *ui);
 
 		for(const NPC &npc : npcs)
 			if(npc.IsLeftBehind(player.GetSystem()))
 			{
-				ui->Push(new Dialog("This is a stop for one of your missions, but you have left a ship behind."));
+				ui->Push(new DialogPanel("This is a stop for one of your missions, but you have left a ship behind."));
 				return false;
 			}
 
@@ -1197,16 +1278,16 @@ bool Mission::Do(Trigger trigger, PlayerInfo &player, UI *ui, const shared_ptr<S
 	// not prevent a mission from being failed or aborted.
 	if(trigger == FAIL)
 	{
-		--player.Conditions()[name + ": active"];
-		++player.Conditions()[name + ": failed"];
+		--player.Conditions()[trueName + ": active"];
+		++player.Conditions()[trueName + ": failed"];
 	}
 	else if(trigger == ABORT)
 	{
-		--player.Conditions()[name + ": active"];
-		++player.Conditions()[name + ": aborted"];
+		--player.Conditions()[trueName + ": active"];
+		++player.Conditions()[trueName + ": aborted"];
 		// Set the failed mission condition here as well for
 		// backwards compatibility.
-		++player.Conditions()[name + ": failed"];
+		++player.Conditions()[trueName + ": failed"];
 	}
 
 	// Don't update any further conditions if this action exists and can't be completed.
@@ -1215,26 +1296,23 @@ bool Mission::Do(Trigger trigger, PlayerInfo &player, UI *ui, const shared_ptr<S
 
 	if(trigger == ACCEPT)
 	{
-		++player.Conditions()[name + ": offered"];
-		++player.Conditions()[name + ": active"];
+		++player.Conditions()[trueName + ": offered"];
+		++player.Conditions()[trueName + ": active"];
 		// Any potential on offer conversation has been finished, so update
-		// the active NPCs for the first time.
+		// the active NPCs for the first time and cache any necessary information.
 		UpdateNPCs(player);
-		if(deadline)
-		{
-			DistanceMap here(player);
-			player.CalculateRemainingDeadline(*this, here);
-		}
+		DistanceMap here(player);
+		player.CacheMissionInformation(*this, here);
 	}
 	else if(trigger == DECLINE)
 	{
-		++player.Conditions()[name + ": offered"];
-		++player.Conditions()[name + ": declined"];
+		++player.Conditions()[trueName + ": offered"];
+		++player.Conditions()[trueName + ": declined"];
 	}
 	else if(trigger == COMPLETE)
 	{
-		--player.Conditions()[name + ": active"];
-		++player.Conditions()[name + ": done"];
+		--player.Conditions()[trueName + ": active"];
+		++player.Conditions()[trueName + ": done"];
 	}
 
 	// "Jobs" should never show dialogs when offered, nor should they call the
@@ -1255,7 +1333,7 @@ bool Mission::Do(Trigger trigger, PlayerInfo &player, UI *ui, const shared_ptr<S
 		it->second.Do(player, ui, this, (destination && isVisible) ? destination->GetSystem() : nullptr,
 			boardingShip, IsUnique());
 	else if(trigger == OFFER && location != JOB)
-		player.MissionCallback(Conversation::ACCEPT);
+		player.MissionCallback(Endpoint::ACCEPT);
 
 	return true;
 }
@@ -1300,6 +1378,15 @@ void Mission::UpdateNPCs(const PlayerInfo &player)
 
 
 
+// Iterate through the timers and progress them if applicable.
+void Mission::StepTimers(PlayerInfo &player, UI &ui)
+{
+	for(MissionTimer &timer : timers)
+		timer.Step(player, ui, *this);
+}
+
+
+
 // Checks if the given ship belongs to one of the mission's NPCs.
 bool Mission::HasShip(const shared_ptr<Ship> &ship) const
 {
@@ -1314,7 +1401,7 @@ bool Mission::HasShip(const shared_ptr<Ship> &ship) const
 
 // If any event occurs between two ships, check to see if this mission cares
 // about it. This may affect the mission status or display a message.
-void Mission::Do(const ShipEvent &event, PlayerInfo &player, UI *ui)
+void Mission::Do(const ShipEvent &event, PlayerInfo &player, UI &ui)
 {
 	if(event.TargetGovernment()->IsPlayer() && !IsFailed())
 	{
@@ -1337,29 +1424,30 @@ void Mission::Do(const ShipEvent &event, PlayerInfo &player, UI *ui)
 				failed |= (it.first == this);
 			if(failed)
 				message = "Your " + event.Target()->DisplayModelName() +
-					" \"" + event.Target()->Name() + "\" has been plundered. ";
+					" \"" + event.Target()->GivenName() + "\" has been plundered. ";
 		}
 
 		if(failed)
 		{
 			hasFailed = true;
 			if(isVisible)
-				Messages::Add(message + "Mission failed: \"" + displayName + "\".", Messages::Importance::Highest);
+				Messages::Add({message + "Mission failed: \"" + displayName + "\".",
+					GameData::MessageCategories().Get("high")});
 		}
 	}
 
 	if((event.Type() & ShipEvent::DISABLE) && event.Target() == player.FlagshipPtr())
-		Do(DISABLED, player, ui);
+		Do(DISABLED, player, &ui);
 
-	// Jump events are only created for the player's flagship.
-	if((event.Type() & ShipEvent::JUMP) && event.Actor())
+	const Ship *flagship = player.Flagship();
+	if((event.Type() & ShipEvent::JUMP) && flagship && event.Actor().get() == flagship)
 	{
 		const System *system = event.Actor()->GetSystem();
 		// If this was a waypoint, clear it.
 		if(waypoints.erase(system))
 		{
 			visitedWaypoints.insert(system);
-			Do(WAYPOINT, player, ui);
+			Do(WAYPOINT, player, &ui);
 		}
 
 		// Perform an "on enter" action for this system, if possible, and if
@@ -1369,17 +1457,13 @@ void Mission::Do(const ShipEvent &event, PlayerInfo &player, UI *ui)
 	}
 
 	for(NPC &npc : npcs)
-		npc.Do(event, player, ui, this, isVisible);
-}
-
-
-
-// Get the internal name used for this mission. This name is unique and is
-// never modified by string substitution, so it can be used in condition
-// variables, etc.
-const string &Mission::Identifier() const
-{
-	return name;
+	{
+		bool isTarget = npc.Do(event, player, ui, this, isVisible);
+		// If this event is a DESTROY or JUMP event for one of the NPCs in this mission, then
+		// recalculate all tracked NPC locations.
+		if((event.Type() & (ShipEvent::JUMP | ShipEvent::DESTROY)) && isTarget)
+			RecalculateTrackedSystems();
+	}
 }
 
 
@@ -1416,7 +1500,7 @@ Mission Mission::Instantiate(const PlayerInfo &player, const shared_ptr<Ship> &b
 	result.overridesCapture = overridesCapture;
 	result.sourceShip = boardingShip.get();
 	result.repeat = repeat;
-	result.name = name;
+	result.trueName = trueName;
 	result.waypoints = waypoints;
 	result.completionFilter = completionFilter;
 	result.markedSystems = markedSystems;
@@ -1555,7 +1639,7 @@ Mission Mission::Instantiate(const PlayerInfo &player, const shared_ptr<Ship> &b
 	if(player.GetPlanet())
 		subs["<origin>"] = player.GetPlanet()->DisplayName();
 	else if(boardingShip)
-		subs["<origin>"] = boardingShip->Name();
+		subs["<origin>"] = boardingShip->GivenName();
 	subs["<planet>"] = result.destination ? result.destination->DisplayName() : "";
 	subs["<system>"] = result.destination ? result.destination->GetSystem()->DisplayName() : "";
 	subs["<destination>"] = subs["<planet>"] + " in the " + subs["<system>"] + " system";
@@ -1590,66 +1674,88 @@ Mission Mission::Instantiate(const PlayerInfo &player, const shared_ptr<Ship> &b
 
 	// Instantiate the NPCs. This also fills in the "<npc>" substitution.
 	string reason;
-	for(auto &&n : npcs)
-		reason = n.Validate(true);
-	if(!reason.empty())
+	for(const NPC &npc : npcs)
 	{
-		Logger::LogError("Instantiation Error: NPC template in mission \""
-			+ Identifier() + "\" uses invalid " + std::move(reason));
-		return result;
+		reason = npc.Validate(true);
+		if(!reason.empty())
+		{
+			Logger::Log("Instantiation Error: NPC template in mission \""
+				+ TrueName() + "\" uses invalid " + std::move(reason), Logger::Level::WARNING);
+			return result;
+		}
 	}
 	for(const NPC &npc : npcs)
 		result.npcs.push_back(npc.Instantiate(player, subs, sourceSystem, result.destination->GetSystem(), jumps, payload));
+	result.hasTrackedNpcs = hasTrackedNpcs;
+
+	// Instantiate the timers.
+	for(const MissionTimer &timer : timers)
+		result.timers.push_back(timer.Instantiate(subs, sourceSystem, jumps, payload));
 
 	// Instantiate the actions. The "complete" action is always first so that
 	// the "<payment>" substitution can be filled in.
-	auto ait = actions.begin();
-	for( ; ait != actions.end(); ++ait)
+	for(const auto &[trigger, action] : actions)
 	{
-		reason = ait->second.Validate();
+		reason = action.Validate();
 		if(!reason.empty())
-			break;
+		{
+			Logger::Log("Instantiation Error: Action \"" + TriggerToText(trigger) + "\" in mission \""
+				+ TrueName() + "\" uses invalid " + std::move(reason), Logger::Level::WARNING);
+			return result;
+		}
 	}
-	if(ait != actions.end())
-	{
-		Logger::LogError("Instantiation Error: Action \"" + TriggerToText(ait->first) + "\" in mission \""
-			+ Identifier() + "\" uses invalid " + std::move(reason));
-		return result;
-	}
-	for(const auto &it : actions)
-		result.actions[it.first] = it.second.Instantiate(subs, sourceSystem, jumps, payload);
+	for(const auto &[trigger, action] : actions)
+		result.actions[trigger] = action.Instantiate(subs, sourceSystem, jumps, payload);
 
-	auto oit = onEnter.begin();
-	for( ; oit != onEnter.end(); ++oit)
+	for(const auto &[system, action] : onEnter)
 	{
-		reason = oit->first->IsValid() ? oit->second.Validate() : "trigger system";
+		reason = system->IsValid() ? action.Validate() : "trigger system";
 		if(!reason.empty())
-			break;
+		{
+			Logger::Log("Instantiation Error: Action \"on enter `" + system->TrueName() + "`\" in mission \""
+				+ TrueName() + "\" uses invalid " + std::move(reason), Logger::Level::WARNING);
+			return result;
+		}
 	}
-	if(oit != onEnter.end())
+	for(const auto &[system, action] : onEnter)
+		result.onEnter[system] = action.Instantiate(subs, sourceSystem, jumps, payload);
+	for(const auto &[planet, action] : onLand)
 	{
-		Logger::LogError("Instantiation Error: Action \"on enter '" + oit->first->TrueName() + "'\" in mission \""
-			+ Identifier() + "\" uses invalid " + std::move(reason));
-		return result;
+		reason = planet->IsValid() ? action.Validate() : "trigger planet";
+		if(!reason.empty())
+		{
+			Logger::Log("Instantiation Error: Action \"on land `" + planet->TrueName() + "`\" in mission \""
+				+ TrueName() + "\" uses invalid " + std::move(reason), Logger::Level::WARNING);
+			return result;
+		}
 	}
-	for(const auto &it : onEnter)
-		result.onEnter[it.first] = it.second.Instantiate(subs, sourceSystem, jumps, payload);
+	for(const auto &[planet, action] : onLand)
+		result.onLand[planet] = action.Instantiate(subs, sourceSystem, jumps, payload);
 
-	auto eit = genericOnEnter.begin();
-	for( ; eit != genericOnEnter.end(); ++eit)
+	for(const MissionAction &action : genericOnEnter)
 	{
-		reason = eit->Validate();
+		reason = action.Validate();
 		if(!reason.empty())
-			break;
-	}
-	if(eit != genericOnEnter.end())
-	{
-		Logger::LogError("Instantiation Error: Generic \"on enter\" action in mission \""
-			+ Identifier() + "\" uses invalid " + std::move(reason));
-		return result;
+		{
+			Logger::Log("Instantiation Error: Generic \"on enter\" action in mission \""
+				+ TrueName() + "\" uses invalid " + std::move(reason), Logger::Level::WARNING);
+			return result;
+		}
 	}
 	for(const MissionAction &action : genericOnEnter)
 		result.genericOnEnter.emplace_back(action.Instantiate(subs, sourceSystem, jumps, payload));
+	for(const MissionAction &action : genericOnLand)
+	{
+		reason = action.Validate();
+		if(!reason.empty())
+		{
+			Logger::Log("Instantiation Error: Generic \"on land\" action in mission \""
+				+ TrueName() + "\" uses invalid " + std::move(reason), Logger::Level::WARNING);
+			return result;
+		}
+	}
+	for(const MissionAction &action : genericOnLand)
+		result.genericOnLand.emplace_back(action.Instantiate(subs, sourceSystem, jumps, payload));
 
 	// Perform substitution in the name and description.
 	result.displayName = Format::Replace(Phrase::ExpandPhrases(displayName), subs);
@@ -1717,13 +1823,13 @@ int Mission::CalculateJumps(const System *sourceSystem)
 
 // Perform an "on enter" MissionAction associated with the current system.
 // Returns true if an action was performed.
-bool Mission::Enter(const System *system, PlayerInfo &player, UI *ui)
+bool Mission::Enter(const System *system, PlayerInfo &player, UI &ui)
 {
 	const auto eit = onEnter.find(system);
 	const auto originalSize = didEnter.size();
 	if(eit != onEnter.end() && !didEnter.contains(&eit->second) && eit->second.CanBeDone(player, IsFailed()))
 	{
-		eit->second.Do(player, ui, this);
+		eit->second.Do(player, &ui, this);
 		didEnter.insert(&eit->second);
 	}
 	// If no specific `on enter` was performed, try matching to a generic "on enter,"
@@ -1732,7 +1838,7 @@ bool Mission::Enter(const System *system, PlayerInfo &player, UI *ui)
 		for(MissionAction &action : genericOnEnter)
 			if(!didEnter.contains(&action) && action.CanBeDone(player, IsFailed()))
 			{
-				action.Do(player, ui, this);
+				action.Do(player, &ui, this);
 				didEnter.insert(&action);
 				break;
 			}
@@ -1742,6 +1848,29 @@ bool Mission::Enter(const System *system, PlayerInfo &player, UI *ui)
 
 
 
+bool Mission::Land(const Planet *planet, PlayerInfo &player, UI &ui)
+{
+	const auto lit = onLand.find(planet);
+	const auto originalSize = didLand.size();
+	if(lit != onLand.end() && !didLand.contains(&lit->second) && lit->second.CanBeDone(player, IsFailed()))
+	{
+		lit->second.Do(player, &ui, this);
+		didLand.insert(&lit->second);
+	}
+	else
+		for(MissionAction &action : genericOnLand)
+			if(!didLand.contains(&action) && action.CanBeDone(player, IsFailed()))
+			{
+				action.Do(player, &ui, this);
+				didLand.insert(&action);
+				break;
+			}
+
+	return didLand.size() > originalSize;
+}
+
+
+
 // For legacy code, contraband definitions can be placed in two different
 // locations, so move that parsing out to a helper function.
 bool Mission::ParseContraband(const DataNode &node)
